@@ -18,9 +18,10 @@
 #include <stdlib.h>
 #include "pdbAll.h"
 
-static const char *L1 = "~cg";
-static const char *L2 = "~lr";
-static const char *MSG[] = { "", "Bad classID ", "Bad groupID " };
+static const char *L1 = "~cgg";
+static const char *L2 = "~lrr";
+static const char *MSG[] = { "", "Bad classID ", "Bad groupID ",
+                             "Bad groupID " };
 
 static const char *fname = 0;
 static int line = -1;
@@ -41,13 +42,15 @@ void pdb_ERROR(const char *msg, const char *item, int val) {
 #include "pdbTraits.h"
 
 template<class tag>
-typename PDBTraits<tag>::item_t *PDB::findItem(tag t, int id) {
+typename PDBTraits<tag>::item_t *PDB::findItem(tag, int id) {
+  typedef typename PDBTraits<tag>::item_t item_t;
+
   typename PDBTraits<tag>::map_t& map = PDBTraits<tag>::getMap(this);
   typename PDBTraits<tag>::map_t::iterator it = map.find(id);
   if ( it != map.end() )
     return (*it).second;
   else {
-    PDBTraits<tag>::item_t* item = new PDBTraits<tag>::item_t(id);
+    item_t* item = new item_t(id);
     map[id] = item;
     PDBTraits<tag>::getItems(this).push_back(item);
     return item;
@@ -55,16 +58,18 @@ typename PDBTraits<tag>::item_t *PDB::findItem(tag t, int id) {
 }
 
 template<class tag>
-typename PDBTraits<tag>::item_t* PDB::findItem(tag t, const string& name, int id) {
+typename PDBTraits<tag>::item_t* PDB::findItem(tag, const string& name, int id) {
+  typedef typename PDBTraits<tag>::item_t item_t;
+
   typename PDBTraits<tag>::map_t& map = PDBTraits<tag>::getMap(this);
   typename PDBTraits<tag>::map_t::iterator it = map.find(id);
   if ( it != map.end() ) {
-    PDBTraits<tag>::item_t *i = (*it).second;
+    item_t *i = (*it).second;
     if ( i->name()[0] ) pdb_ERROR("Duplicate", i->desc(), id);
     i->name(name);
     return i;
   } else {
-    PDBTraits<tag>::item_t* item = new PDBTraits<tag>::item_t(name, id);
+    item_t* item = new item_t(name, id);
     map[id] = item;
     PDBTraits<tag>::getItems(this).push_back(item);
     return item;
@@ -72,7 +77,7 @@ typename PDBTraits<tag>::item_t* PDB::findItem(tag t, const string& name, int id
 }
 
 template<class tag>
-void PDB::finalCheck(tag t) {
+void PDB::finalCheck(tag) {
   typename PDBTraits<tag>::map_t& map = PDBTraits<tag>::getMap(this);
   typename PDBTraits<tag>::vec_t& vec = PDBTraits<tag>::getVec(this);
   for (typename PDBTraits<tag>::map_t::const_iterator mit = map.begin();
@@ -85,7 +90,7 @@ void PDB::finalCheck(tag t) {
 }
 
 template<class tag>
-void PDB::markDuplicates(tag t, PDB& other) {
+void PDB::markDuplicates(tag, PDB& other) {
   typename PDBTraits<tag>::vec_t& vecOld = PDBTraits<tag>::getVec(this);
   typename PDBTraits<tag>::vec_t& vecNew = PDBTraits<tag>::getVec(&other);
   typename PDBTraits<tag>::map_t& mapOld = PDBTraits<tag>::getMap(this);
@@ -179,6 +184,7 @@ bool PDB::merge(char *other_fname) {
     markDuplicates(croutineTag(), other);
     markDuplicates(templateTag(), other);
     markDuplicates(macroTag(), other);
+    markDuplicates(pragmaTag(), other);
     markDuplicates(namespaceTag(), other);
 
     // update/merge callTree / fileTree / classTree
@@ -293,19 +299,19 @@ bool PDB::getLocation(const char *value, char **ptr,
   while ( *value == ' ' ) ++value;
 
   if ( value[0] == 's' && value[1] == 'o' ) {
-    // looke like valid location
+    // looks like valid location
     int f = strtol(value+3, ptr, 10);
     line = strtol((*ptr)+1, ptr, 10);
     col  = strtol((*ptr)+1, ptr, 10);
     file = findItem(fileTag(), f);
   } else if (  value[0]=='N' && value[1]=='U' && value[2]=='L' ) {
-    // looke like valid NUL location
-    *ptr += 9;
-    file = 0;
+    // looks like valid NUL location
+    *ptr = const_cast<char*>(value) + 8;
+    file = NULL;
     line = col = 0;
   } else if ( value[0] == '\0' ) {
     // nothing there => handle like NUL location
-    file = 0;
+    file = NULL;
     line = col = 0;
   } else {
     // everything else is an error
@@ -315,6 +321,28 @@ bool PDB::getLocation(const char *value, char **ptr,
   return true;
 }
 
+bool PDB::getStmt(const char *value, char **ptr, int& st) {
+  // skip leading blanks
+  while ( *value == ' ' ) ++value;
+
+  if ( value[0] == 's' && value[1] == 't' ) {
+    // looks like valid statement
+    st = strtol(value+3, ptr, 10);
+  } else if (  value[0]=='N' && value[1]=='A' ) {
+    // looks like valid NA statement
+    *ptr = const_cast<char*>(value) + 2;
+    st = -1;
+  } else if ( value[0] == '\0' ) {
+    // nothing there => handle like NA statement
+    st = -1;
+  } else {
+    // everything else is an error
+    pdb_ERROR("Bad statementID ", value);
+    return false;
+  }
+  return true;
+}
+
 const int INBUF_SIZE=2048;
 
 bool PDB::read(char *filename) {
@@ -337,6 +365,7 @@ bool PDB::read(char *filename) {
   pdbTemplate *tptr = 0;
   pdbTemplateItem *tiptr = 0;
   pdbMacro *mptr = 0;
+  pdbPragma *pptr = 0;
   pdbType *yptr = 0;
   pdbNamespace *nptr = 0;
   pdbFile *fptr = 0;
@@ -357,15 +386,25 @@ bool PDB::read(char *filename) {
 
   // check magic number
   if ( !istr.getline(inbuf, INBUF_SIZE) ||
-       ((strcmp(inbuf, "<PDB 1.0>")!=0) && (strcmp(inbuf, "<PDB 2.0>")!=0)) ) {
+       ((strcmp(inbuf, "<PDB 1.0>")!=0) &&
+	(strcmp(inbuf, "<PDB 2.0>")!=0) &&
+	(strcmp(inbuf, "<PDB 3.0>")!=0)) ) {
     cerr << "ERROR: Wrong magic number" << endl;
     return false;
   } else {
     versNum = inbuf[5] - '0';
-    // Version 1 only supported C/C++
-    if ( versNum == 1 ) lang = LA_C_or_CXX;
-    // skip empty line
+
+    // read optional language specifier and skip empty line
     istr.getline(inbuf, INBUF_SIZE);
+    if ( strncmp(inbuf, "lang", 4) == 0 ) {
+      line = 2;
+      lang = toLang(inbuf+5);
+      istr.getline(inbuf, INBUF_SIZE);
+      line = 4;
+    } else if ( versNum == 1 ) {
+      // Version 1 only supported C/C++
+      lang = LA_C_or_CXX;
+    }
   }
   
   while ( istr.getline(inbuf, INBUF_SIZE) || istr.gcount() ) {
@@ -379,6 +418,7 @@ bool PDB::read(char *filename) {
       rptr = 0;
       tptr = 0;
       mptr = 0;
+      pptr = 0;
       nptr = 0;
       fptr = 0;
       yptr = 0;
@@ -458,6 +498,8 @@ bool PDB::read(char *filename) {
         if ( language() == LA_C_or_CXX ) lang = LA_CXX;
       } else if ( fst=='m' && snd=='a' ) {
         iptr = mptr = findItem(macroTag(), name, num);
+      } else if ( fst=='p' && snd=='r' ) {
+        iptr = pptr = findItem(pragmaTag(), name, num);
       } else if ( fst=='n' && snd=='a' ) {
         iptr = fiptr = nptr = findItem(namespaceTag(), name, num);
         if ( language() == LA_C_or_CXX ) lang = LA_CXX;
@@ -550,6 +592,9 @@ bool PDB::read(char *filename) {
                 arg.templateArg(findItem(templateTag(), r));
 	      } else pdb_ERROR("Bad templateID ", ptr);
               break;
+            case pdbTemplateArg::TA_NA:
+              // ignore
+              break;
             }
             tiptr->addSpeclArgument(arg);
           }
@@ -674,6 +719,28 @@ bool PDB::read(char *filename) {
           break;
 
         // ==== File Attributes =========================================
+        case SCOM:     // ---- comment
+	  if ( value[0] == 'c' && value[1] == 'o' ) {
+	    pdbComment* cmt = fptr->addComment(strtol(value+3, &ptr, 10));
+	    ptr = strtok(ptr+1, " ");
+	    cmt->kind(PDB::toLang(ptr));
+	    ptr = strtok(NULL, "");
+	    if ( getLocation(ptr, &ptr, f, l, c) ) {
+              cmt->cmtBegin(pdbLoc(f, l, c));
+              if ( getLocation(ptr, &ptr, f, l, c) ) {
+                cmt->cmtEnd(pdbLoc(f, l, c));
+	      }
+	      string txt(ptr+1);
+	      while ( inbuf[INBUF_SIZE-1] != '\n' ) {
+                inbuf[INBUF_SIZE-1] = '\n';
+                if (istr.fail()) istr.clear(istr.rdstate() & ~ios::failbit);
+                istr.getline(inbuf, INBUF_SIZE);
+                txt += inbuf;
+              }
+              cmt->text(txt);
+	    }
+	  } else pdb_ERROR("Bad commentID ", value);
+	  break;
         case SINC:     // ---- File Include
 	  if ( value[0] == 's' && value[1] == 'o' ) {
             if (fptr) {
@@ -702,7 +769,63 @@ bool PDB::read(char *filename) {
           }
           break;
 
+        // ==== Pragma Attributes =======================================
+        case PKIND:    // ---- Pragma Kind
+          if (pptr) pptr->kind(value);
+          break;
+        case PPOS:     // ---- Pragma Position
+          if (pptr) {
+            if ( getLocation(value, &ptr, f, l, c) ) {
+              pptr->prBegin(pdbLoc(f, l, c));
+              if ( getLocation(ptr, &ptr, f, l, c) )
+                pptr->prEnd(pdbLoc(f, l, c));
+	    }
+	  }
+          break;
+        case PTEXT:    // ---- Pragma Text
+          if (pptr) {
+            string txt(value);
+            while ( inbuf[INBUF_SIZE-1] != '\n' ) {
+              inbuf[INBUF_SIZE-1] = '\n';
+              if (istr.fail()) istr.clear(istr.rdstate() & ~ios::failbit);     
+              istr.getline(inbuf, INBUF_SIZE);
+              txt += inbuf;
+            }
+            pptr->text(txt);
+          }
+          break;
+
+	
         // ==== Routine Attributes ======================================
+        case RBODY:    // ---- Statement which makes up routine body
+	  if ( value[0] == 's' && value[1] == 't' ) {
+            if (rptr) rptr->body(rptr->addStmt(strtol(value+3, &ptr, 10)));
+	  } else pdb_ERROR("Bad statementID ", value);
+          break;
+        case RSTMT:    // ---- Routine Statement
+	  if ( value[0] == 's' && value[1] == 't' ) {
+            pdbStmt* s = rptr->addStmt(strtol(value+3, &ptr, 10));
+            ptr = strtok(ptr+1, " ");
+            s->kind(pdbStmt::toStmt(ptr));
+            ptr = strtok(NULL, "");
+            if ( getLocation(ptr, &ptr, f, l, c) ) {
+              s->stmtBegin(pdbLoc(f, l, c));
+              if ( getLocation(ptr, &ptr, f, l, c) ) {
+                s->stmtEnd(pdbLoc(f, l, c));
+                int sno = -1;
+                if ( getStmt(ptr, &ptr, sno) ) {
+                  s->nextStmt(rptr->addStmt(sno));
+                  if ( getStmt(ptr, &ptr, sno) ) {
+                    s->downStmt(rptr->addStmt(sno));
+                    if ( getStmt(ptr, &ptr, sno) ) {
+                      s->extraStmt(rptr->addStmt(sno));
+                    }
+                  }
+                }
+              }
+            }
+	  } else pdb_ERROR("Bad statementID ", value);
+          break;
         case RCALL:    // ---- Calls Routine
 	  if ( value[0] == 'r' && value[1] == 'o' ) {
             int r = strtol(value+3, &ptr, 10);
@@ -857,7 +980,7 @@ bool PDB::read(char *filename) {
                 int r = strtol(ptr+3, &ptr, 10);
                 arg.type(findItem(typeTag(), r));
 	      } else pdb_ERROR("Bad typeID ", ptr);
-              if ( ptr = strtok(NULL, " ") )
+              if ( (ptr = strtok(NULL, " ")) )
                 arg.defaultType(getTypeOrGroup(ptr));
               }
               break;
@@ -868,7 +991,7 @@ bool PDB::read(char *filename) {
 	      } else pdb_ERROR("Bad typeID ", ptr);
               ptr = strtok(NULL, " ");
               arg.name(ptr);
-              if ( ptr = strtok(NULL, " ") ) arg.defaultValue(ptr);
+              if ( (ptr = strtok(NULL, " ")) ) arg.defaultValue(ptr);
               }
               break;
             case pdbTemplateArg::TA_TEMPL: {
@@ -876,7 +999,7 @@ bool PDB::read(char *filename) {
                 int r = strtol(ptr+3, &ptr, 10);
                 arg.templateArg(findItem(templateTag(), r));
 	      } else pdb_ERROR("Bad templateID ", ptr);
-              if ( ptr = strtok(NULL, " " ) ) {
+              if ( (ptr = strtok(NULL, " " )) ) {
 	        if ( ptr[0] == 't' && ptr[1] == 'e' ) {
                   int r = strtol(ptr+3, &ptr, 10);
                   arg.defaultTemplateArg(findItem(templateTag(), r));
@@ -884,6 +1007,9 @@ bool PDB::read(char *filename) {
               }
               }
               break;
+            case pdbTemplateArg::TA_NA:
+              // ignore
+              break;
             }
             tptr->addArgument(arg);
           }
@@ -907,6 +1033,9 @@ bool PDB::read(char *filename) {
                 arg.templateArg(findItem(templateTag(), r));
 	      } else pdb_ERROR("Bad templateID ", ptr);
               break;
+            case pdbTemplateArg::TA_NA:
+              // ignore
+              break;
             }
             tptr->addSpeclArgument(arg);
           }
@@ -1005,16 +1134,25 @@ bool PDB::read(char *filename) {
             ptr = strtok(value, " ");
             if ( version() < 2 ) {
               ptr = strtok(NULL, " ");
-              yptr->addArgument(getTypeOrGroup(value), "-",
+              yptr->addArgument(getTypeOrGroup(value), "-", NULL, 0, 0,
                        (ptr && ptr[0]=='T') ? pdbArg::ARG_DEF : pdbArg::ARG_NA);
             } else {
               ptr = strtok(NULL, " ");
               string name(ptr);
-              int f = pdbArg::ARG_NA;
-              while ( (ptr = strtok(NULL, " ")) != NULL ) {
-                f |= pdbArg::toArg(ptr);
+              if ( ((ptr = strtok(NULL, "")) != NULL) &&
+                   (ptr[0] == 's' || ptr[0] == 'N')) {
+                getLocation(ptr, &ptr, f, l, c);
+              } else {
+                f = 0;
+                l = c = 0;
               }
-              yptr->addArgument(getTypeOrGroup(value), name, f);
+              int flg = pdbArg::ARG_NA;
+              char* p = ptr;
+              while ( (ptr = strtok(p, " ")) != NULL ) {
+                flg |= pdbArg::toArg(ptr);
+                p = NULL;
+              }
+              yptr->addArgument(getTypeOrGroup(value), name, f, l, c, flg);
             }
           }
           break;
@@ -1085,6 +1223,7 @@ bool PDB::read(char *filename) {
 
         // ==== Invalid Attributes= =====================================
         case NOATTR:
+          // ignore
           break;
 
         default:
@@ -1113,6 +1252,7 @@ bool PDB::read(char *filename) {
   finalCheck(moduleTag());
   finalCheck(templateTag());
   finalCheck(macroTag());
+  finalCheck(pragmaTag());
   finalCheck(namespaceTag());
   // in case of errors above stop here because these often crash routine checks
   if (!ok) return false;
