@@ -18,6 +18,7 @@ static bool Aflag = false;
 static bool Gflag = false;
 static bool Mflag = false;
 static bool Nflag = false;
+static bool Pflag = false;
 static bool Rflag = false;
 static bool Sflag = false;
 static bool Tflag = false;
@@ -266,39 +267,67 @@ void printPos(ostream& ostr, const pdbLoc& l) {
   }
 }
 
-void printPositions(ostream& ostr, const pdbFatItem* fi) {
-  pdbLoc l;
-
-  ostr << "headPosition:      ";
-  printPos(ostr,  fi->headBegin());
-  l = fi->headEnd();
-  if ( l.file() && fi->headBegin().file() == l.file() )
-    ostr << " -> (" << l.line() << "," << l.col() << ")";
+void printRange(ostream& ostr,const pdbLoc& l1, const pdbLoc& l2) { 
+  printPos(ostr, l1);
+  if ( l2.file() && l1.file() == l2.file() )
+    ostr << " -> (" << l2.line() << "," << l2.col() << ")";
   else
-    printPos(ostr, l);
+    printPos(ostr, l2);
+}
+
+void printPositions(ostream& ostr, const pdbFatItem* fi) {
+  ostr << "headPosition:      ";
+  printRange(ostr, fi->headBegin(), fi->headEnd());
   ostr << "\n";
   ostr << "bodyPosition:      ";
-  printPos(ostr,  fi->bodyBegin());
-  l = fi->bodyEnd();
-  if ( l.file() && fi->bodyBegin().file() == l.file() )
-    ostr << " -> (" << l.line() << "," << l.col() << ")";
-  else
-    printPos(ostr, l);
+  printRange(ostr, fi->bodyBegin(), fi->bodyEnd());
   ostr << "\n";
 }
 
+void printStmt(ostream& ostr, const pdbStmt& s) {
+  const pdbStmt* s1;
+
+  ostr << "ST#" << s.id() << " " << pdbStmt::toName(s.kind());
+  printRange(ostr, s.stmtBegin(), s.stmtEnd());
+  if ((s1 = s.nextStmt()) != NULL)
+    ostr << " ST#" << s1->id();
+  else
+    ostr << " NA";
+  if ((s1 = s.downStmt()) != NULL)
+    ostr << " ST#" << s1->id();
+  else
+    ostr << " NA";
+  if ((s1 = s.extraStmt()) != NULL)
+    ostr << " ST#" << s1->id();
+}
+
 void printLo(ostream& ostr, const pdbLoc& l) {
   ostr << "location:          ";
   printPos(ostr, l);
   ostr << "\n";
 }
 
+void printComment(ostream& ostr, const pdbComment& c) {
+  ostr << "CO#" << c.id() << " " << PDB::toName(c.kind());
+  printRange(ostr, c.cmtBegin(), c.cmtEnd());
+  ostr << " " << c.text();
+
+}
+
 void printSo(ostream& ostr, const pdbFile *f) {
   ostr << "SO#" << f->id() << " " << f->name() << "\n";
+  if ( f->isSystemFile() )
+    ostr << "isSystemInclude:    TRUE\n";
   for (pdbFile::incvec::const_iterator it=f->includes().begin();
        it!=f->includes().end(); ++it)
     ostr << "includes:           SO#"
          << (*it)->id() << " " << (*it)->name() << "\n";
+  for (pdbFile::cmtvec::const_iterator ct=f->comments().begin();
+       ct!=f->comments().end(); ++ct) {
+    ostr << "comment:            ";
+    printComment(ostr, **ct);
+    ostr << "\n";
+  }
   ostr << endl;
 }
 
@@ -312,7 +341,7 @@ void printItem(ostream& ostr, const pdbItem *i) {
   if ( const pdbGroup* gptr = i->parentGroup() ) {
     ostr << "group:              GR#"
          << gptr->id() << " " << gptr->name() << "\n";
-    ostr << "access:             " << toName(i->access()) << "\n";
+    ostr << "access:             " << ::toName(i->access()) << "\n";
   }
   if ( const pdbNamespace* nptr = i->parentNSpace() ) {
     ostr << "namespace:          NA#"
@@ -324,13 +353,13 @@ void printTeArg(ostream& ostr, const pdbTemplateArg& ta) {
   const pdbType* ty;
   const pdbTemplate *te;
 
-  ostr << toName(ta.kind());
+  ostr << ::toName(ta.kind());
   if ( ta.kind() == pdbTemplateArg::TA_TYPE ) {
     if ( ta.isSpecialization() ) {
       printType(ostr, " ", ta.type(), "");
     } else {
       ostr << " TY#" << ta.type()->id() << " " << ta.type()->name();
-      if ( ty = ta.defaultType() ) printType(ostr, " ", ty, "");
+      if ( (ty = ta.defaultType()) ) printType(ostr, " ", ty, "");
     }
   } else if ( ta.kind() == pdbTemplateArg::TA_NONTYPE ) {
     if ( ta.isSpecialization() ) {
@@ -343,7 +372,7 @@ void printTeArg(ostream& ostr, const pdbTemplateArg& ta) {
   } else if ( ta.kind() == pdbTemplateArg::TA_TEMPL ) {
     ostr << " TE#" << ta.templateArg()->id() << " "
          << ta.templateArg()->name();
-    if ( te = ta.defaultTemplateArg() )
+    if ( (te = ta.defaultTemplateArg()) )
       ostr << " TE#" << te->id() << " " << te->name();
   }
   ostr << "\n";
@@ -373,7 +402,7 @@ void printTy(ostream& ostr, const pdbType *t) {
   pdbType::boundvec b;
 
   printItem(ostr, t);
-  ostr << "kind:               " << toName(t->kind()) << "\n";
+  ostr << "kind:               " << ::toName(t->kind()) << "\n";
   switch ( t->kind() ) {
   case pdbItem::TY_INT:
   case pdbItem::TY_BOOL:
@@ -381,7 +410,7 @@ void printTy(ostream& ostr, const pdbType *t) {
   case pdbItem::TY_FINT:
   case pdbItem::TY_FLOGIC:
   case pdbItem::TY_FCHAR:
-    ostr << "integerKind:        " << toName(t->integerType()) << "\n";
+    ostr << "integerKind:        " << ::toName(t->integerType()) << "\n";
     if ( t->isSigned() ) ostr << "explicitlySigned:   TRUE" << "\n";
     if ( t->kind() == pdbItem::TY_FCHAR ) {
       int n = t->numElements();
@@ -392,7 +421,7 @@ void printTy(ostream& ostr, const pdbType *t) {
     }
     break;
   case pdbItem::TY_ENUM: {
-    ostr << "integerKind:        " << toName(t->integerType()) << "\n";
+    ostr << "integerKind:        " << ::toName(t->integerType()) << "\n";
     for (pdbType::enumvec::const_iterator eit=t->enumeration().begin();
          eit!=t->enumeration().end(); ++eit)
       ostr << "enumerationValue:   " << (*eit).id()
@@ -402,7 +431,7 @@ void printTy(ostream& ostr, const pdbType *t) {
   case pdbItem::TY_FLOAT:
   case pdbItem::TY_FFLOAT:
   case pdbItem::TY_FCMPLX:
-    ostr << "floatKind:          " << toName(t->floatType()) << "\n";
+    ostr << "floatKind:          " << ::toName(t->floatType()) << "\n";
     break;
   case pdbItem::TY_PTR:
   case pdbItem::TY_FPTR:
@@ -418,7 +447,7 @@ void printTy(ostream& ostr, const pdbType *t) {
     break;
   case pdbItem::TY_FARRAY:
     printType(ostr, "elementType:        ", t->elementType());
-    ostr << "shape:              " << toName(t->shape()) << "\n";
+    ostr << "shape:              " << ::toName(t->shape()) << "\n";
     ostr << "rank:               " << t->rank() << "\n";
     b = t->bounds();
     if ( b.size() ) {
@@ -447,7 +476,7 @@ void printTy(ostream& ostr, const pdbType *t) {
     if ( q.size() ) {
       ostr << "qualifiers:        ";
       for (pdbType::qualvec::iterator it=q.begin(); it!=q.end(); ++it)
-        ostr << " " << toName(*it);
+        ostr << " " << ::toName(*it);
       ostr << "\n";
     }
     break;
@@ -458,10 +487,11 @@ void printTy(ostream& ostr, const pdbType *t) {
          ait!=t->arguments().end(); ++ait) {
       printType(ostr, "argument:           ", ait->type(), "");
       ostr << " " << ait->name();
-      if ( ait->intentIn()   ) {ostr << " " << toName(pdbArg::ARG_IN);}
-      if ( ait->intentOut()  ) {ostr << " " << toName(pdbArg::ARG_OUT);}
-      if ( ait->isOptional() ) {ostr << " " << toName(pdbArg::ARG_OPT);}
-      if ( ait->hasDefault() ) {ostr << " " << toName(pdbArg::ARG_DEF);}
+      printPos(ostr, *ait);
+      if ( ait->intentIn()   ) {ostr << " " << ::toName(pdbArg::ARG_IN);}
+      if ( ait->intentOut()  ) {ostr << " " << ::toName(pdbArg::ARG_OUT);}
+      if ( ait->isOptional() ) {ostr << " " << ::toName(pdbArg::ARG_OPT);}
+      if ( ait->hasDefault() ) {ostr << " " << ::toName(pdbArg::ARG_DEF);}
       ostr << '\n';
     }
     if ( t->kind() == pdbItem::TY_FUNC ) {
@@ -496,9 +526,9 @@ void printGf(ostream& ostr, const pdbGroupField *g) {
     if ( gptr->kind() == pdbItem::GR_FMODULE && !g->location().file() )
       ostr << "  group:              GR#"
            << gptr->id() << " " << gptr->name() << "\n";
-    ostr << "  access:             " << toName(g->access()) << "\n";
+    ostr << "  access:             " << ::toName(g->access()) << "\n";
   }
-  ostr << "  kind:               " << toName(g->kind()) << "\n";
+  ostr << "  kind:               " << ::toName(g->kind()) << "\n";
   if ( g->type() ) printType(ostr, "  type:               ", g->type());
   printTemplItem(ostr, g, "  ");
   if ( g->isStaticConst() ) ostr << "  isStaticConst:      TRUE\n";
@@ -508,9 +538,9 @@ void printGf(ostream& ostr, const pdbGroupField *g) {
 
 void printGroup(ostream& ostr, const pdbGroup *g) {
   printItem(ostr, g);
-  ostr << "kind                " << toName(g->kind()) << "\n";
+  ostr << "kind                " << ::toName(g->kind()) << "\n";
   printTemplItem(ostr, g, "");
-  const pdbGroup::fieldvec d = g->dataMembers();
+  const pdbGroup::fieldvec& d = g->dataMembers();
   for (pdbGroup::fieldvec::const_iterator dt=d.begin(); dt!=d.end(); ++dt)
      printGf(ostr, *dt);
   printPositions(ostr, g);
@@ -519,21 +549,21 @@ void printGroup(ostream& ostr, const pdbGroup *g) {
 void printClass(ostream& ostr, const pdbClass *c) {
   printGroup(ostr, c);
   if ( c->isABC() ) ostr << "isAbstractBaseClass:  TRUE\n";
-  const pdbClass::basevec b = c->baseClasses();
+  const pdbClass::basevec& b = c->baseClasses();
   for (pdbClass::basevec::const_iterator bt=b.begin(); bt!=b.end(); ++bt) {
      ostr << "baseClass:          "
-          << toName((*bt)->virtuality())
-          << " " << toName((*bt)->access())
+          << ::toName((*bt)->virtuality())
+          << " " << ::toName((*bt)->access())
           << " GR#" << (*bt)->base()->id() << " " << (*bt)->base()->name()
           << " ";
      printPos(ostr, **bt);
      ostr << "\n";
   }
-  const pdbClass::classvec s = c->derivedClasses();
+  const pdbClass::classvec& s = c->derivedClasses();
   for (pdbClass::classvec::const_iterator st=s.begin(); st!=s.end(); ++st)
      ostr << "derivedClass:       GR#"
           << (*st)->id() << " "  << (*st)->name() << "\n";
-  const pdbClass::friendclassvec fc = c->friendClasses();
+  const pdbClass::friendclassvec& fc = c->friendClasses();
   for (pdbClass::friendclassvec::const_iterator fct=fc.begin();
        fct!=fc.end(); ++fct) {
      ostr << "friendClass:        GR#" << (*fct)->friendClass()->id()
@@ -541,7 +571,7 @@ void printClass(ostream& ostr, const pdbClass *c) {
      printPos(ostr, **fct);
      ostr << "\n";
   }
-  const pdbClass::friendfuncvec fr = c->friendRoutines();
+  const pdbClass::friendfuncvec& fr = c->friendRoutines();
   for (pdbClass::friendfuncvec::const_iterator frt=fr.begin();
        frt!=fr.end(); ++frt) {
      ostr << "friendRoutine:      RO#" << (*frt)->friendFunc()->id()
@@ -549,7 +579,7 @@ void printClass(ostream& ostr, const pdbClass *c) {
      printPos(ostr, **frt);
      ostr << "\n";
   }
-  const pdbClass::methodvec f = c->methods();
+  const pdbClass::methodvec& f = c->methods();
   for (pdbClass::methodvec::const_iterator ft=f.begin(); ft!=f.end(); ++ft) {
      ostr << "method:             RO#"
           << (*ft)->func()->id() << " " << (*ft)->func()->name() << " ";
@@ -561,7 +591,7 @@ void printClass(ostream& ostr, const pdbClass *c) {
 
 void printModule(ostream& ostr, const pdbModule *m) {
   printGroup(ostr, m);
-  const pdbModule::modfuncvec f = m->funcMembers();
+  const pdbModule::modfuncvec& f = m->funcMembers();
   for (pdbModule::modfuncvec::const_iterator ft=f.begin(); ft!=f.end(); ++ft) {
      ostr << "moduleFunc:         RO#"
           << (*ft)->func()->id() << " " << (*ft)->func()->name() << " ";
@@ -575,33 +605,44 @@ void printRo(ostream& ostr, const pdbRoutine *r) {
   pdbItem::rspec_t s;
   pdbItem::routine_t k;
   pdbItem::link_t l;
+  const pdbStmt* st;
 
   printItem(ostr, r);
   if ( r->signature() )
     ostr << "signature:          TY#" << r->signature()->id() << " "
                           << r->signature()->name() << "\n";
-  if ( (l = r->linkage()) != pdbItem::RO_NA )
-    ostr << "linkage:            " << toName(l) << "\n";
+  if ( (l = r->linkage()) != pdbItem::LK_NA )
+    ostr << "linkage:            " << ::toName(l) << "\n";
   if ( (k = r->kind()) != pdbItem::RO_NA )
-    ostr << "kind:               " << toName(k) << "\n";
+    ostr << "kind:               " << ::toName(k) << "\n";
   if ( (s = r->specialKind()) != pdbItem::RS_NA )
-    ostr << "specialKind:        " << toName(s) << "\n";
+    ostr << "specialKind:        " << ::toName(s) << "\n";
   printTemplItem(ostr, r, "");
-  const pdbRoutine::callvec c = r->callees();
+  const pdbRoutine::callvec& c = r->callees();
   for (pdbRoutine::callvec::const_iterator ct=c.begin(); ct!=c.end(); ++ct) {
     const pdbCallee *func = *ct;
     ostr << "calls:              RO#" << func->call()->id() << " "
-         << func->call()->name() << " " << toName(func->virtuality()) << " ";
+         << func->call()->name() << " " << ::toName(func->virtuality()) << " ";
     printPos(ostr, *func);
     ostr << "\n";
   }
   printPositions(ostr, r);
+  if ( (st = r->body()) )
+    ostr << "routineBody:        ST#" << st->id() << "\n";
+  const pdbRoutine::stmtvec& stmts = r->statements();
+  for (int i=0; i<stmts.size(); ++i) {
+    if ( stmts[i] ) {
+      ostr << "bodyStatement:      ";
+      printStmt(ostr, *(stmts[i]));
+      ostr << "\n";
+    }
+  }
   if ( r->firstExecStmtLocation().file() ) {
     ostr << "firstExecStmt:     ";
     printPos(ostr, r->firstExecStmtLocation());
     ostr << "\n";
   }
-  const pdbRoutine::locvec lr = r->returnLocations();
+  const pdbRoutine::locvec& lr = r->returnLocations();
   for (pdbRoutine::locvec::const_iterator lt=lr.begin(); lt!=lr.end(); ++lt) {
     ostr << "returnStmt:        ";
     printPos(ostr, **lt);
@@ -614,7 +655,7 @@ void printCRo(ostream& ostr, const pdbCRoutine *r) {
 
   printRo(ostr, r);
   if ( (v = r->virtuality()) != pdbItem::VI_NO )
-    ostr << "virtuality:         " << toName(v) << "\n";
+    ostr << "virtuality:         " << ::toName(v) << "\n";
   if ( r->covariantReturnVirtualOverride() )
     ostr << "covariantReturnVirtualOverride:  TRUE\n";
   if ( r->isCompilerGenerated() ) ostr << "isCompilerGenerated:  TRUE\n";
@@ -626,7 +667,7 @@ void printCRo(ostream& ostr, const pdbCRoutine *r) {
 
 void printFRo(ostream& ostr, const pdbFRoutine *r) {
   printRo(ostr, r);
-  const pdbRoutine::locvec lr = r->stopLocations();
+  const pdbRoutine::locvec& lr = r->stopLocations();
   for (pdbRoutine::locvec::const_iterator lt=lr.begin(); lt!=lr.end(); ++lt) {
     ostr << "stopStmt:           ";
     printPos(ostr, **lt);
@@ -640,10 +681,10 @@ void printFRo(ostream& ostr, const pdbFRoutine *r) {
     ostr << "aliasFor:           RO#" << r->isAliasFor()->id() << " "
          << r->isAliasFor()->name() << "\n";
   }
-  const pdbFRoutine::implvec im = r->implementedBy();
-  for (pdbFRoutine::implvec::const_iterator lt=im.begin(); lt!=im.end(); ++lt)
-    ostr << "implementedBy:      RO#" << (*lt)->id() << " "
-         << (*lt)->name() << "\n";
+  const pdbFRoutine::implvec& im = r->implementedBy();
+  for (pdbFRoutine::implvec::const_iterator it=im.begin(); it!=im.end(); ++it)
+    ostr << "implementedBy:      RO#" << (*it)->id() << " "
+         << (*it)->name() << "\n";
   if ( r->hasExplicitInterface() )
     ostr << "explicitInterface:  TRUE" << "\n";
   if ( r->isRecursive() ) ostr << "isRecursive:        TRUE" << "\n";
@@ -656,11 +697,11 @@ void printTe(ostream& ostr, const pdbTemplate *t) {
   const pdbTemplate *te;
 
   printItem(ostr, t);
-  ostr << "kind:               " << toName(t->kind()) << "\n";
+  ostr << "kind:               " << ::toName(t->kind()) << "\n";
   if ( !(t->text() == "") ) ostr << "text:               " << t->text() << "\n";
-  if ( te = t->declaration() )
+  if ( (te = t->declaration()) )
     ostr << "declaration:        TE#" << te->id() << " " << te->name() << "\n";
-  if ( te = t->definition() )
+  if ( (te = t->definition()) )
     ostr << "definition:         TE#" << te->id() << " " << te->name() << "\n";
 
   const pdbTemplate::targvec& a = t->arguments();
@@ -669,16 +710,16 @@ void printTe(ostream& ostr, const pdbTemplate *t) {
     printTeArg(ostr, *it);
   }
   const pdbTemplate::targvec& s = t->speclArguments();
-  for (pdbTemplate::targvec::const_iterator it=s.begin(); it!=s.end(); ++it) {
+  for (pdbTemplate::targvec::const_iterator st=s.begin(); st!=s.end(); ++st) {
     ostr << "specializationParameter: ";
-    printTeArg(ostr, *it);
+    printTeArg(ostr, *st);
   }
 
   if ( const pdbCRoutine* ro = t->funcProtoInst() ) {
     ostr << "prototypeInstance:  RO#" << ro->id() << " " << ro->name() << "\n";
   } else if ( const pdbClass* cl = t->classProtoInst() ) {
     ostr << "prototypeInstance:  GR#" << cl->id() << " " << cl->name() << "\n";
-  } else if ( ty = t->statMemType() ) {
+  } else if ( (ty = t->statMemType()) ) {
     printType(ostr, "ttype ", ty);
   }
 
@@ -703,31 +744,41 @@ void printNa(ostream& ostr, const pdbNamespace *n) {
 
 void printMa(ostream& ostr, const pdbMacro *m) {
   printItem(ostr, m);
-  ostr << "kind:               " << toName(m->kind()) << "\n";
+  ostr << "kind:               " << ::toName(m->kind()) << "\n";
   ostr << "text:               " << m->text() << "\n";
   ostr << endl;
 }
 
+void printPr(ostream& ostr, const pdbPragma *p) {
+  printItem(ostr, p);
+  ostr << "kind:               " << p->kind() << "\n";
+  ostr << "position:          ";
+  printRange(ostr, p->prBegin(), p->prEnd());
+  ostr << "\n";
+  ostr << "text:               " << p->text() << "\n";
+  ostr << endl;
+}
+
 void printPDB(PDB& pdb, ostream& ostr) {
   if ( Aflag || Sflag ) {
-    PDB::filevec f = pdb.getFileVec();
+    PDB::filevec& f = pdb.getFileVec();
     for (PDB::filevec::iterator fi = f.begin(); fi != f.end(); ++fi)
       printSo(ostr, *fi);
   }
 
   if ( Aflag || Yflag ) {
-    PDB::typevec t = pdb.getTypeVec();
+    PDB::typevec& t = pdb.getTypeVec();
     for (PDB::typevec::iterator ty = t.begin(); ty != t.end(); ++ty)
       printTy(ostr, *ty);
   }
 
   if ( Aflag || Gflag ) {
     if ( pdb.language() & PDB::LA_C_or_CXX ) {
-      PDB::classvec c = pdb.getClassVec();
+      PDB::classvec& c = pdb.getClassVec();
       for (PDB::classvec::iterator cl = c.begin(); cl != c.end(); ++cl)
         printClass(ostr, *cl);
     } else if ( pdb.language() & PDB::LA_FORTRAN ) {
-      PDB::modulevec m = pdb.getModuleVec();
+      PDB::modulevec& m = pdb.getModuleVec();
       for (PDB::modulevec::iterator mo = m.begin(); mo != m.end(); ++mo)
         printModule(ostr, *mo);
     }
@@ -735,33 +786,39 @@ void printPDB(PDB& pdb, ostream& ostr) {
 
   if ( Aflag || Rflag ) {
     if ( pdb.language() & PDB::LA_C_or_CXX ) {
-      PDB::croutinevec r = pdb.getCRoutineVec();
+      PDB::croutinevec& r = pdb.getCRoutineVec();
       for (PDB::croutinevec::iterator ro = r.begin(); ro != r.end(); ++ro)
         printCRo(ostr, *ro);
     } else if ( pdb.language() & PDB::LA_FORTRAN ) {
-      PDB::froutinevec r = pdb.getFRoutineVec();
+      PDB::froutinevec& r = pdb.getFRoutineVec();
       for (PDB::froutinevec::iterator ro = r.begin(); ro != r.end(); ++ro)
         printFRo(ostr, *ro);
     }
   }
 
   if ( Aflag || Tflag ) {
-    PDB::templatevec u = pdb.getTemplateVec();
+    PDB::templatevec& u = pdb.getTemplateVec();
     for (PDB::templatevec::iterator te = u.begin(); te != u.end(); ++te)
       printTe(ostr, *te);
   }
 
   if ( Aflag || Nflag ) {
-    PDB::namespacevec n = pdb.getNamespaceVec();
+    PDB::namespacevec& n = pdb.getNamespaceVec();
     for (PDB::namespacevec::iterator na = n.begin(); na != n.end(); ++na)
       printNa(ostr, *na);
   }
 
   if ( Aflag || Mflag ) {
-    PDB::macrovec m = pdb.getMacroVec();
+    PDB::macrovec& m = pdb.getMacroVec();
     for (PDB::macrovec::iterator ma = m.begin(); ma != m.end(); ++ma)
       printMa(ostr, *ma);
   }
+
+  if ( Aflag || Pflag ) {
+    PDB::pragmavec& p = pdb.getPragmaVec();
+    for (PDB::pragmavec::iterator pr = p.begin(); pr != p.end(); ++pr)
+      printPr(ostr, *pr);
+  }
 }
 
 int main(int argc, char *argv[]) {
@@ -769,12 +826,13 @@ int main(int argc, char *argv[]) {
   bool errflag = argc < 2;
   char *outfile = 0;
 
-  while ( (ch = getopt(argc, argv, "AGMNRSTYco:")) != -1 ) {
+  while ( (ch = getopt(argc, argv, "AGMNPRSTYco:")) != -1 ) {
     switch (ch) {
     case 'A': Aflag = true;     break;
     case 'G': Gflag = true;     break;
     case 'M': Mflag = true;     break;
     case 'N': Nflag = true;     break;
+    case 'P': Pflag = true;     break;
     case 'R': Rflag = true;     break;
     case 'S': Sflag = true;     break;
     case 'T': Tflag = true;     break;
@@ -786,7 +844,7 @@ int main(int argc, char *argv[]) {
   }
   if ( errflag ) {
     cerr << "usage: " << argv[0]
-         << " [-c|o outfile] [-AGMNRSTY] pdbfile" << endl;
+         << " [-c|o outfile] [-AGMNPRSTY] pdbfile" << endl;
     return 1;
   }
 
@@ -802,7 +860,7 @@ int main(int argc, char *argv[]) {
   } else 
     ostr = &cout;
 
-  if ( Aflag || Gflag || Mflag || Nflag ||
+  if ( Aflag || Gflag || Mflag || Nflag || Pflag ||
        Rflag || Sflag || Tflag || Yflag )
     printPDB(pdb, *ostr);
   else
