@@ -109,7 +109,7 @@ bool filenamedataCmp( filenamedata* f1, filenamedata* f2) {
    } else {
       return f1->name < f2->name;
    }
-};
+}
 
 
 //----------------------------------------------------------------------
@@ -1715,8 +1715,6 @@ static bool processFile(PDB& pdb, filenamedata* f,
                      strcpy( fname, (*it)->includeform.c_str() );
                   } 
 
-                  int flen = strlen(fname);
-
                   if ( strncmp(inbuf+pos+1, fname, len)==0 ) {
 
                      ostr << "<A HREF=\"";
@@ -1971,7 +1969,7 @@ static bool processFile(PDB& pdb, filenamedata* f,
                                        nextLine == itemvec[curr-1]->line &&
                                        nextCol   == itemvec[curr-1]->col );
 
-                        for (unsigned i=0; i<len; ++i) {
+                        for (int i=0; i<len; ++i) {
                            out(ostr, inbuf[pos++]);
                         }
                         while ( numAnchor-- ) ostr << "</A>";
