@@ -11,7 +11,9 @@
 
 
 #ifndef __APPLE__
-#ifndef PDT_WINDOWS
+#ifdef PDT_WINDOWS
+#include "include\wchar.h"
+#else
 #include "/usr/include/wchar.h"
 #endif /* PDT_WINDOWS */
 #endif /* __APPLE__ */
@@ -92,10 +94,10 @@ const char __lookuptable[] = {
 
 typedef struct{
         long double x;
-        } LONGDOUBLE;
+        } sgiLONGDOUBLE;
 typedef struct {
         double x;
-} DOUBLE;
+} sgiDOUBLE;
 #define _LEADBYTE   0x8000
 #define isleadbyte(_c)  (_c & _LEADBYTE)
 #define ASCII       32
@@ -103,9 +105,9 @@ typedef struct {
 #define ___decimal_point "."
 typedef void (* PFV)(void);
 #define _cfltcvt_tab[6] {void, void, void, void, void, void}
-typedef void (* PF5)(LONGDOUBLE*, char*, int, int, int);
+typedef void (* PF5)(sgiLONGDOUBLE*, char*, int, int, int);
 typedef void (* PF2)(int, char*, char*);
-typedef void (* PF0)(DOUBLE*, char*, int, int, int);
+typedef void (* PF0)(sgiDOUBLE*, char*, int, int, int);
 typedef void (* PF3)(char*);
 typedef void (* PF1)(char*);
 #define _cropzeros(a)       (*((PF1)_cfltcvt_tab[1]))(a)
@@ -191,9 +193,11 @@ size_t wcsftime(wchar_t *s, size_t maxsize, const wchar_t *format, const struct
 #ifdef  __cplusplus
 }
 #endif
-#elif defined(__cray) 
+#elif (defined(__cray) || defined (hpux) || defined(__hiuxmpp))
 
+#ifndef _crayx1
 typedef int mbstate_t;
+#endif /* _crayx1 */
 
 
 
@@ -224,7 +228,14 @@ size_t wcsrtombs(char *dst, const wchar_t **src, size_t n, mbstate_t *pstate);
 size_t mbrlen(const char* s, size_t n, mbstate_t *type);
 
 /* Other fixes for PDT */
+#ifndef _crayx1
 typedef const int * wctrans_t;
+#else /* CRAY X1 */
+#ifndef _WCTRANS_T
+#   define _WCTRANS_T
+    typedef unsigned long	wctrans_t;
+#endif
+#endif /* CRAY X1 */
 
 wint_t btowc(int c); 
 int wctob(wint_t c);
@@ -326,6 +337,6 @@ size_t wcsxfrm(wchar_t *, const wchar_t *, size_t);
 
 
 
-#endif /* sgi | linux | cray | apple */
+#endif /* sgi | linux | cray | hitachi | apple */
 
 #endif /* __KAI_WCHAR_H */
